function [tt,xx,uu,g] = mpc_renewerg_ODE3(N,T,x0,mpciterations,ls,myprint)
% Renewenerg example for MPC with dynamics as ODE
%
% Parameters:
%
%    N  = horizon for open loop optimization
%    T  = time step (sampling interval length)
%    x0 = initial value
%    mpciterations = number of iterations for simulation
%    ls = color of closed loop trajectory (in MATLAB linespec format)
%    myprint = control of graphical output
%              1: only closed loop trajectory
%              2: open and closed loop trajectories
%
% Output:
%
%    tt: time grid of closed loop solution
%    xx: states of closed loop solution
%    uu: control of closed loop solution
%    g:  stage cost at final point of closed loop solution
%
% Example call:
%
%    mpc_renewenerg_ODE3(6,0.5,[1, 5, 1],30,'-k',2); (plot with predictions)
%
%    mpc_renewenerg_ODE3(6,0.5,[1, 5, 1],119,'-k',1); (plot without predictions)
% 
%
% note: needs nmpc.m, available from www.nmpc-book.com/matlab_nmpc.html 
%       the file nmpc.m must be contained in the same directory as this
%       file


    t0            = 0.0;                    % initial time
    u0            = 0.5*ones(2,N);          % initial guess for control
                                            % in first NMPC step
    tol_opt       = 1e-5;                   % tolerance for optimizer
    opt_option    = 0;                      % choice of optimizer
    iprint        = 5;                      % controls verbosity
    type          = 'differential equation';% type of dynamics
    atol_ode_real = 1e-10;                  % tolerances for ODE solver in
    rtol_ode_real = 1e-10;                  % simulation of MPC closed loop
    atol_ode_sim  = 1e-2;                   % tolerances for ODE solver in
    rtol_ode_sim  = 1e-2;                   % optimization

    % model parameters are specified in the subroutines, below
    
    % discount factor
    global lambda;
    lambda = exp(-T*0.03);  
        
    % some global variables for internal use
    global gN;
    gN = N;
    
    global gprint;
    gprint = myprint;
    
    global guu;
    guu = [];
    
    global gtt;
    gtt = [ t0 ];
    
    global gxx;
    gxx = [ x0 ];
    
    global gls;
    gls = ls;
    
    % start of the nmpc routine
    nmpc(@runningcosts, @terminalcosts, @constraints, ...
         @terminalconstraints, @linearconstraints, @system, ...
         mpciterations, N, T, t0, x0, u0, ...
         tol_opt, opt_option, ...
         type, atol_ode_real, rtol_ode_real, atol_ode_sim, rtol_ode_sim, ...
         iprint, @printHeader, @printClosedloopData, @plotTrajectories);

    % for output of some statistical values 
    % (time, state, control along closed loop trajectory, 
    %  stage cost at final point of closed loop trajectory)
    tt = gtt;
    xx = gxx;
    uu = guu;
    
    length(uu)
    g = runningcosts(tt(length(tt)),xx(length(xx),:),uu(length(uu),:));
    
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Definition of the NMPC functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function cost = runningcosts(t, x, u)

    
    global lambda;

    zet   = 0.5;
    sigm  = 2.0;
    Mnull = 1.0;
    
    gamma = 1.0;


    % discount factor lambda is specified above
    %cost = -lambda^t*(u(2)^(1-sigm))* ...
    %                 (x(3)-Mnull)^(-zet*(1-sigm)-1)/(1-sigm);
    
    % simplified variant
    cost = -lambda^t*(log(u(2))-gamma*(x(3)-Mnull)^2);

end

function cost = terminalcosts(t, x)
    cost = 0.0;
end

function [c,ceq] = constraints(t, x, u)
    
    % state constraints for n=0..N-1
    c(1) = -x(1) + 0.0;
    c(2) =  x(1) - 5.0;
    c(3) = -x(2) - 0.0;
    c(4) =  x(2) - 5.0;
    c(5) = -x(3) - 0.0;
    c(6) =  x(3) - 5.0;
    ceq = [];
end

function [c,ceq] = terminalconstraints(t, x)
    global gconstr;
    
    % state constraints for n=N
    c(1) = -x(1) + 0.0;
    c(2) =  x(1) - 5.0;
    c(3) = -x(2) - 0.0;
    c(4) =  x(2) - 5.0;
    c(5) = -x(3) - 0.0;
    c(6) =  x(3) - 5.0;
    ceq = [];
end

function [A, b, Aeq, beq, lb, ub] = linearconstraints(t, x, u)
    A   = [];
    b   = [];
    Aeq = [];
    beq = [];
    % upper and lower bound on control values
    lb(1) =  0.0;
    ub(1) =  0.5;
    lb(2) =  0.0;
    ub(2) = 10.0;
end

function y = system(t, x, u, T)
    
    % Parameters
    rho1  = 0.05;
    deltt = 0.05;
    kapp  = 2.0;
    alff  = 0.5;
    AAA   = 1.0;
    An    = 1.0; %1.8; 1=normal 2.0; 2.50;%%1
    a1    = 0.1;
    zet   = 0.5;
    sigm  = 2.0;
    Ap    = 1.0;
    Rstar = 3.0;
    alf1  = 0.0;
    bet1  = 0.2;%0.1;0.05
    mu    = 0.05;
    Mnull = 1.0;
    
    % ODE model
    y  = zeros(3,1);
    uu=max(0.05,u(1));%0.005; 0.01 %0.05;
    E  = max(0,An*x(1)+Ap*uu);
    YY = AAA*E^alff;

    y(1) = YY-u(2)-deltt*x(1)-a1*uu;%u(1);      % = k, red
    y(2) = alf1*x(2)*(Rstar-x(1))-uu;%u(1);     % = R, blue
    y(3) = bet1*uu-mu*(x(3)-kapp*Mnull);  % = M, black
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Definition of output format, called from within the NMPC routine at
% each step of the iteration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function printHeader()
    fprintf('   k  | u(k)  uu        x(k)                cost     Time\n');
    fprintf('-------------------------------------------------------\n');
end

function printClosedloopData(mpciter, u, x, t_Elapsed)
    
    fprintf(' %3d  | %+5.2f %+5.2f   %+5.2f %+5.2f %+5.2f   %+5.2f  %+7.3f', mpciter, ...
            u(1,1), u(2,1), x(1), x(2), x(3), t_Elapsed); %runningcosts(mpciter, x, u(:,1)), t_Elapsed);
       
end

function plotTrajectories(dynamic, system, T, t0, x0, u, ...
                          atol_ode, rtol_ode, type)
    global gN;
    global gprint;
    global gtt;
    global gxx;
    global guu;
    global gls;
    
    gtt = [gtt; t0+T];
    
    if (gprint>=1)
        figure(1);
        hold on;  
    end;
        x = x0;
        for k=1:gN
            [x,t_intermediate, x_intermediate] = ...
                dynamic(system, T, t0+T*(k-1), x, u(:,k), ...
                             atol_ode, rtol_ode, type);
            if (k==1)
                if (gprint>=1)
                    plot(t_intermediate, x_intermediate(:,1), 'r-');
                    plot(t_intermediate, x_intermediate(:,2), 'b-');
                    plot(t_intermediate, x_intermediate(:,3), 'k-');
                end;
                gxx = [gxx; x];
                guu = [guu; u(:,1)'];
            else
                if (gprint>=2)
                    plot(t_intermediate,x_intermediate(:,1),'--r');
                    plot(t_intermediate,x_intermediate(:,2),'--b');
                    plot(t_intermediate,x_intermediate(:,3),'--k');
                end;
            end 
        end
        if (gprint>=1)
            if (gprint>=2)
               % title(['open loop trajectories (dashed) and closed loop trajectory (solid)']);
            else
               % title(['closed loop trajectory (solid)']);
            end;
            xlabel('t');
            ylabel('x_1,x_2,x_3');
            grid on;
        end;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
